#include <cerrno>
#include <cstring>
#include <sstream>
#include <iostream>
#include "xunix.h"

xunix::exception::exception(const std::string& prefix) : std::exception()
{
  init(prefix);
}

xunix::exception::~exception() throw()
{
  // empty
}

const char* xunix::exception::what() const throw()
{
  return _what.c_str();
}

int xunix::exception::error_number() const throw()
{
  return _error_number;
}

void xunix::exception::init(const std::string& prefix)
{
  // Save errno asap.
  _error_number = errno;
  std::ostringstream ostrm;
  if( prefix.length() > 0 ) {
    ostrm << prefix << ": ";
  }
  ostrm << strerror(_error_number);
  _what += ostrm.str();
}

int xunix::open(const char* filename, int flags, mode_t mode)
{
  int rv = ::open(filename, flags, mode);
  if( rv == -1 ) {
    std::ostringstream ostrm;
    ostrm << "open(\"" << filename << "\")";
    throw( xunix::exception(ostrm.str()) );
  }
  return rv;
}

void xunix::close(int fd)
{
  if( ::close(fd) == -1 ) {
    std::ostringstream ostrm;
    ostrm << "close(" << fd << ")";
    throw( xunix::exception(ostrm.str()) );
  }
}

void xunix::fcntl(int fd, int cmd, struct flock* lock_flock)
{
  if( ::fcntl(fd, cmd, lock_flock) == -1 ) {
    throw( xunix::exception("fcntl") );
  }
}

void xunix::sigaction(int signum,
                     const struct sigaction* act,
                     struct sigaction* oldact)
  throw(xunix::exception)
{
  if( ::sigaction(signum, act, oldact) == -1 ) {
    std::ostringstream ostrm;
    ostrm << "sigaction(" << signum << ")";
    throw( xunix::exception(ostrm.str()) );
  }
}

void xunix::sigprocmask(int how, const sigset_t* sig_set, sigset_t* old_sig_set)
{
  if( ::sigprocmask(how, sig_set, old_sig_set) == -1 ) {
    throw( xunix::exception("sigprocmask") );
  }
}

void xunix::sigsuspend(const sigset_t* mask)
{
  ::sigsuspend(mask);
}

int xunix::sigwaitinfo(const sigset_t* sig_set, siginfo_t* sig_info)
{
  int rv = ::sigwaitinfo(sig_set, sig_info);
  if( rv == -1 ) {
    throw( xunix::exception("sigwaitinfo") );
  }
  return rv;
}

void xunix::sigemptyset(sigset_t* sigset)
{
  if( ::sigemptyset(sigset) == - 1 ) {
    throw( xunix::exception("sigemptyset") );
  }
}

void xunix::sigfillset(sigset_t* sigset)
{
  if( ::sigfillset(sigset) == -1 ) {
    throw( xunix::exception("sigfillset") );
  }
}

void xunix::sigaddset(sigset_t* sigset, int signum)
{
  if( ::sigaddset(sigset, signum) == -1 ) {
    throw( xunix::exception("sigaddset") );
  }
}

void xunix::sigdelset(sigset_t* sigset, int signum)
{
  if( ::sigdelset(sigset, signum) == -1 ) {
    throw( xunix::exception("sigdelset") );
  }
}

int xunix::sigismember(const sigset_t* sigset, int signum)
{
  int rv = ::sigismember(sigset, signum);
  if( rv == -1 ) {
    throw( xunix::exception("sigismember") );
  }
  return rv;
}

void xunix::pause(void)
{
  ::pause();
}
